--[[
Created by Michael
--]]

local Grid2Layout= Grid2:GetModule("Grid2Layout")
local Grid2Frame= Grid2:GetModule("Grid2Frame")
local texture
local layoutName
local colCount
local rowCount
local colColors
local frames

function Grid2Layout:ShowFrames(enabled)
	for type, headers in pairs(self.groups) do
		for i = 1, self.indexes[type] do
			local g = headers[i]
			if enabled then
				g:Show()
			else	
				g:Hide()
			end	
		end
	end
end

local function LayoutGetTestFrame(i)
	if frames then 
		if frames[i] then return frames[i] end	
	else
		frames= {}
	end	

	local f= CreateFrame("Frame", nil, Grid2Layout.frame)
	f:SetBackdrop({
		bgFile = texture, tile = false, tileSize = 0,
		edgeFile = "Interface\\Addons\\Grid2\\white16x16", edgeSize = 1,
		insets = {left = 1, right = 1, top = 1, bottom = 1},
	})

	frames[i]= f
	return f
end

local LayoutGetVectors
do
	local vectors= {
	 ["TOPLEFT"]    = { 1,0, 0, 1, 0, 0},
	 ["TOPRIGHT"]   = {-1,0, 0, 1, 1, 0},
	 ["BOTTOMLEFT"] = { 1,0, 0,-1, 0, 1},
	 ["BOTTOMRIGHT"]= {-1,0, 0,-1, 1, 1},
	}
	LayoutGetVectors= function(anchor, horizontal, cols, rows)
		local ux,uy,vx,vy,px,py= unpack(vectors[anchor])
		if horizontal then
			return vx,vy,ux,uy,px*(rows-1),py*(cols-1),rows,cols
		else
			return ux,uy,vx,vy,px*(cols-1),py*(rows-1),cols,rows	
		end
	end
end

local function LayoutHide(restoreRealLayout)
	if layoutName then
		local framesCount= colCount*rowCount	
		for i=1,framesCount do
			frames[i]:Hide()	
		end
		if restoreRealLayout then
			layoutName= nil
			Grid2Layout:ShowFrames(true)
			Grid2Layout:UpdateSize()
		end	
	end	
end

local LayoutLoad
do
	local colorsTable= {
		["partypet"]= { r=0,g=1,b=0, a=0.35},
		["raidpet"] = { r=0,g=1,b=0, a=0.35},
		["spacer"]  = { r=0,g=0,b=0, a=0},
	}
	LayoutLoad= function(name)
		if layoutName then
			LayoutHide(false)
		end
		if not texture then
			local media = LibStub("LibSharedMedia-3.0", true)
			texture = media and media:Fetch("statusbar", Grid2Frame.db.profile.frameTexture) or "Interface\\Addons\\Grid2\\gradient32x32"		
		end
		colColors= {}
		local layout = Grid2Layout.layoutSettings[name]
		if layout then
			local defaults = layout.defaults or {} 
			colCount= 0
			rowCount= 0
			local col= 1
			for i, l in ipairs(layout) do
				local unitPerColumn= l.unitsPerColumn or defaults.unitPerColumn or 5
				local maxColumns= l.maxColumns or defaults.maxColumns or 1
				colCount= colCount + maxColumns
				rowCount= max(rowCount,unitPerColumn)
				local c= l.type and colorsTable[l.type] or RAID_CLASS_COLORS[ CLASS_SORT_ORDER[((i-1)%#CLASS_SORT_ORDER)+1] ] 
				for j=1,maxColumns do
					colColors[col]= { c.r*0.5,c.g *0.5,c.b*0.5, c.a or 0.75}
					col= col + 1
				end	
			end
			layoutName= name
			return true
		end
	end
end

local function LayoutRefresh()
	if not layoutName then return end

    Grid2Layout:ShowFrames(false)	

	local settings= Grid2Layout.db.profile
	local frameBorder = Grid2Frame.db.profile.frameBorder
	local width,height= Grid2Frame:GetFrameSize()
	local frameLevel= Grid2Layout.frame:GetFrameLevel() + 25
	local Spacing= settings.Spacing
	local Padding= settings.Padding
    local ux,uy,vx,vy,px,py,realCols,realRows= LayoutGetVectors(settings.groupAnchor, settings.horizontal, colCount, rowCount)
	
	local i= 1
	for nx=0,colCount-1 do
		local r,g,b,a= unpack(colColors[nx+1])
		for ny=0,rowCount-1 do
			local x= nx*ux + ny*vx + px  
			local y= nx*uy + ny*vy + py  
			local offx= Spacing + x * (width + Padding) 
			local offy= Spacing + y * (height + Padding)
			local frame= LayoutGetTestFrame(i)
			frame:ClearAllPoints()
			frame:SetPoint("TOPLEFT", Grid2LayoutFrame, "TOPLEFT", offx, -offy )
			frame:SetSize( width,height )
			frame:SetBackdropColor( r,g,b,a )
			frame:SetBackdropBorderColor( 0,0,0,a)
			frame:SetFrameLevel(frameLevel)
			frame:Show()
			r,g,b= r*0.7, g*0.7, b*0.7
			i= i + 1
		end	
	end	
	Grid2Layout.frame:SetSize( Spacing*2 + realCols * (width+Padding) - Padding, Spacing*2 + realRows * (height+Padding) - Padding )
end	

local function LayoutEnable(self,name)
	if name and name ~= layoutName then
		if LayoutLoad(name) then
			LayoutRefresh()
			return true
		end 
	else
		LayoutHide(true)
		return false
	end	
end

Grid2Options.LayoutTestEnable= LayoutEnable
Grid2Options.LayoutTestRefresh= LayoutRefresh
